// Joris Navarro Hello board skecth for Facacademy 2018 Barcelona - Week 9 Embedded programming
//
// hello.joris.84.c
//

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include <Arduino.h>

// library for serial communication with ATtiny24/44/84
#include "SoftwareSerial/SoftwareSerial.h"

// library from adafruit to use neopixels (aka ws2812b)
#include "Adafruit_NeoPixel/Adafruit_NeoPixel.h"

#define BUTTON_PIN   3    
#define PIXEL_PIN    7
#define PIXEL_COUNT 3

#define RX_PIN  0
#define TX_PIN  1

// for ATtiny84 with 8Mhz internal clock use NEO_KHZ400 !
Adafruit_NeoPixel strip = Adafruit_NeoPixel(PIXEL_COUNT, PIXEL_PIN, NEO_GRB + NEO_KHZ400);

// init serial communication
SoftwareSerial serial(RX_PIN, TX_PIN);

bool oldState = HIGH;
int showType = 0;

void sendToSerial(String text){
  serial.println(text);
}

/*
 * Setup 
 */
void setup() {
  pinMode(BUTTON_PIN, INPUT_PULLUP);
  
  // initialize serial communication at 9600 bits per second:
  serial.begin(9600);
  
  strip.begin();
  strip.show(); // Initialize all pixels to 'off'

  rainbowCycle(10);
  
  serial.println("Ready");
}

void processButton(){
  
    // Get current button state.
  bool newState = digitalRead(BUTTON_PIN);

  // Check if state changed from high to low (button press).
  if (newState == LOW && oldState == HIGH) {
    // Short delay to debounce button.
    delay(20);
    // Check if button is still low after debounce.
    newState = digitalRead(BUTTON_PIN);
    if (newState == LOW) {
      showType++;
      if (showType > 9)
        showType=0;      
      switchMode(showType);
    }
  }

  // Set the last button state to the old state.
  oldState = newState;
}

// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<strip.numPixels(); i++) {
    strip.setPixelColor(i, c);
    strip.show();
    delay(wait);
  }
}

void rainbow(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256; j++) {
    for(i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel((i+j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256*5; j++) { // 5 cycles of all colors on wheel
    for(i=0; i< strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

//Theatre-style crawling lights.
void theaterChase(uint32_t c, uint8_t wait) {
  for (int j=0; j<10; j++) {  //do 10 cycles of chasing
    for (int q=0; q < 3; q++) {
      for (int i=0; i < strip.numPixels(); i=i+3) {
        strip.setPixelColor(i+q, c);    //turn every third pixel on
      }
      strip.show();

      delay(wait);

      for (int i=0; i < strip.numPixels(); i=i+3) {
        strip.setPixelColor(i+q, 0);        //turn every third pixel off
      }
    }
  }
}

//Theatre-style crawling lights with rainbow effect
void theaterChaseRainbow(uint8_t wait) {
  int modulo = 2; // used to be 3
  for (int j=0; j < 256; j++) {     // cycle all 256 colors in the wheel
    for (int q=0; q < 3; q++) {
      for (int i=0; i < strip.numPixels(); i=i+modulo) {
        strip.setPixelColor(i+q, Wheel( (i+j) % 255));    //turn every modulo pixel on
      }
      strip.show();

      delay(wait);

      for (int i=0; i < strip.numPixels(); i=i+modulo) {
        strip.setPixelColor(i+q, 0);        //turn every modulo pixel off
      }
    }
  }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if(WheelPos < 85) {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if(WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}

void switchMode(int cmd) {
  serial.print("go to mode ");
  serial.println(cmd);
  switch(cmd){
    case 0: colorWipe(strip.Color(0, 0, 0), 50);    // Black/off
            sendToSerial("off");
            break;
    case 1: colorWipe(strip.Color(255, 0, 0), 50);  // Red
            break;
    case 2: colorWipe(strip.Color(0, 255, 0), 50);  // Green
            break;
    case 3: colorWipe(strip.Color(0, 0, 255), 50);  // Blue
            break;
    case 4: theaterChase(strip.Color(127, 127, 127), 50); // White
            break;
    case 5: theaterChase(strip.Color(127,   0,   0), 50); // Red
            break;
    case 6: theaterChase(strip.Color(  0,   0, 127), 50); // Blue
            break;
    case 7: rainbow(20);
            break;
    case 8: rainbowCycle(20);
            break;
    case 9: theaterChaseRainbow(50);
            break;
    default:
            return;
  }
}

void readSerial(){
  int value =0, input = 0;  
  switch (serial.available()) {
        /* Serial.available() is the number of bytes waiting.  Convert from 
         * ASCII val to an int.  Intentional switch-case fall through below. 
        */ 
        case 3:
          value = serial.read();
          value -= 48;
          value *= 10;
        case 2:
          value += serial.read();
          value -= 48;
          value *= 10;
        case 1:
          input = 0;
          input = serial.read();
          delay(10);       
          value += input;
          break;
        default:        
          // If >3 chars, just clear out the incoming buffer. 
          while (serial.available())
            serial.read();
          return;      
      }      
  processSerialInput(input);
}

void processSerialInput(int value){  
  int mode;
  switch (value) {
    case 'a':
      mode = 0;
      break;
    case 'b':
      mode = 1;     
      break;
    case 'c':
      mode = 2;   
      break;
    case 'd':
      mode = 3;     
      break;
    case 'e':
      mode = 4;    
      break;
    case 'f':
      mode = 5;   
      break;
    case 'g':
      mode = 6;    
      break;
    case 'h':
      mode = 7;    
      break;
    case 'i':
      mode = 8;    
      break;
    case 'j':
      mode = 9;    
      break;    
    default:  
      mode = 0;
      break;  
  }
  switchMode(mode); 
}

/*
 * Loop
 */
void loop() {
  processButton();
  readSerial();
  delay(500);
}


int main(void) {

    setup();

    while(1){
        loop();
    }
}
